/************************************************************************/
/* Copyright (c) 2003-2005 by SAS Institute Inc., Cary, NC 27513, USA   */
/*                       All Rights Reserved.                           */   
/*                                                                      */        
/* Name:    mo_create_user                                              */
/*                                                                      */
/* Purpose: Create a MO user definition                                 */
/*                                                                      */
/* Input:   username             - MO username                          */
/*          password             - MO password                          */
/*          name                 - user's name            (optional)    */
/*          job_title            - user's job title       (optional)    */
/*          description          - description            (optional)    */
/*          email                - user's email address   (optional)    */
/*          phone                - user's phone number    (optional)    */
/*          manage_project_input - Y=yes, N=no                          */
/*          manage_user_profiles - Y=yes, N=no                          */
/*          manage_projects      - Y=yes, N=no                          */
/*          profile_create_by    - user who created this profile        */
/*                                                                      */
/* Output:  &_rc = return code                                          */
/*                                                                      */  
/* Return Codes:                                                        */  
/*                                                                      */  
/*          100   generic error                                         */
/*          102   invalid or missing input parameter value              */
/*          103   I/O error updating a metadata table                   */
/*          104   I/O error creating a metadata table                   */
/*          105   I/O error creating a directory                        */
/*          106   error obtaining lock                                  */
/*          107   SQL error                                             */
/*          108   could not generate unique identifer                   */
/*          110   username not unique                                   */
/*          200   completed with abnormal condition                     */
/*                                                                      */
/* HISTORY:                                                             */
/* 30Jan04 S0237007 Creation (dwh)                                      */  
/* 05Aug04 S0261023 Create mo_opt_config.sas (dwh)                      */
/* 20Aug04 S0265220 Remove email check (dwh)                            */
/*                                                                      */
/* MO 4.1 Hotfix:                                                       */
/* 17Feb05 S0284290 Use authenticated username (dwh)                    */
/*                                                                      */
/************************************************************************/

%macro mo_create_user(username              = 
                     ,password              =
                     ,name                  =
                     ,job_title             =
                     ,description           =
                     ,email                 =
                     ,phone                 =
                     ,manage_project_input  =
                     ,manage_user_profiles  =
                     ,manage_projects       =
                     ,profile_created_by    =
                     ,_rc      				=
                     );

   %local macname;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);
	
   %if %bquote(&_rc) eq %str() %then %do;                                                                                                  
      %local rc;                                                                                                                      
      %let _rc = rc;                                                                                                                    
   %end;                                                                                                                                   
   %else %global &_rc;                                                                                                                     
   %let &_rc = 0;       


   %*-------------------------------------------------------------------;                                                                                                                            
   %* Validate parms for benefit of administrator running this in batch ;
   %*-------------------------------------------------------------------;      
   %if (%bquote(&username) eq %str()) %then %do;
      %put %mo_error No username was specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;   
   %if (%length(&username) gt 32) %then %do;
      %put %mo_error Username must be 32 characters or fewer.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;   
   %if (%length(&name) gt 64) %then %do;
      %put %mo_error Name must be 64 characters or fewer.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end; 
   %if (%length(&job_title) gt 64) %then %do;
      %put %mo_error Job title must be 64 characters or fewer.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end; 
   %if (%length(&description) gt 256) %then %do;
      %put %mo_error Description must be 256 characters or fewer.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end; 
   %if (%length(&email) gt 64) %then %do;
      %put %mo_error Email address must be 64 characters or fewer.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end; 
   %if (%length(&phone) gt 64) %then %do;
      %put %mo_error Phone must be 64 characters or fewer.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
    %end;
   %if (%bquote(&manage_project_input) eq %str()) %then %do;
      %put %mo_error No manage_project_input specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
    %end;
   %if (%length(&manage_project_input) gt 1) %then %do;
      %put %mo_error Manage_project_input must have a length of 1 character.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %if (%upcase(%bquote(&manage_project_input)) ne Y and %upcase(%bquote(&manage_project_input)) ne N) %then %do;
      %put %mo_error Manage_project_input must be set to Y or N.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %if (%bquote(&manage_user_profiles) eq %str()) %then %do;
      %put %mo_error No manage_user_profiles specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
    %end;
   %if (%length(&manage_user_profiles) gt 1) %then %do;
      %put %mo_error Manage_user_profiles must have a length of 1 character.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %if (%bquote(%upcase(&manage_user_profiles)) ne Y and %bquote(%upcase(&manage_user_profiles)) ne N) %then %do;
      %put %mo_error Manage_user_profiles must be set to Y or N.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %if (%bquote(&manage_projects) eq %str()) %then %do;
      %put %mo_error No manage_projects specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
    %end;
   %if (%length(&manage_projects) gt 1) %then %do;
      %put %mo_error Manage_projects must have a length of 1 character.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %if (%bquote(%upcase(&manage_projects)) ne Y and %bquote(%upcase(&manage_projects)) ne N) %then %do;
      %put %mo_error Manage_projects must be set to Y or N.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %if (%bquote(&profile_created_by) eq %str()) %then %do;
      %put %mo_error No profile_created_by was specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
    %end;
   %if (%length(&profile_created_by) gt 64) %then %do;
      %put %mo_error Profile_created_by must be 64 characters or fewer.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end; 
  

   %*--------------------------------;                                                                                                                            
   %* Create MO user metadata tables ;
   %*--------------------------------;       
   %if not %sysfunc(exist(mometa.users)) %then %do;
      %mo_create_tables(lib=mometa,table=users,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.USERS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.users_notify)) %then %do;
      %mo_create_tables(lib=mometa,table=users_notify,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.USERS_NOTIFY metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.users_share)) %then %do;
      %mo_create_tables(lib=mometa,table=users_share,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.USERS_SHARE metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.users_noshare)) %then %do;
      %mo_create_tables(lib=mometa,table=users_noshare,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.USERS_NOSHARE metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.uid)) %then %do;
      %mo_create_tables(lib=mometa,table=uid,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.UID metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.project_input)) %then %do;
      %mo_create_tables(lib=mometa,table=project_input,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.PROJECT_INPUT metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.projects)) %then %do;
      %mo_create_tables(lib=mometa,table=projects,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.PROJECTS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.scenarios)) %then %do;
      %mo_create_tables(lib=mometa,table=scenarios,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.SCENARIOS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.optimization_jobs)) %then %do;
      %mo_create_tables(lib=mometa,table=optimization_jobs,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.OPTIMIZATION_JOBS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.market_segments)) %then %do;
      %mo_create_tables(lib=mometa,table=market_segments,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.MARKET_SEGMENTS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.campaigns)) %then %do;
      %mo_create_tables(lib=mometa,table=campaigns,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.CAMPAIGNS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.campaign_constraint_rhs)) %then %do;
      %mo_create_tables(lib=mometa,table=campaign_constraint_rhs,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.CAMPAIGN_CONSTRAINT_RHS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.communications)) %then %do;
      %mo_create_tables(lib=mometa,table=communications,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.COMMUNICATIONS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.comm_numeric_attributes)) %then %do;
      %mo_create_tables(lib=mometa,table=comm_numeric_attributes,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.COMM_NUMERIC_ATTRIBUTES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.comm_numeric_attribute_values)) %then %do;
      %mo_create_tables(lib=mometa,table=comm_numeric_attribute_values,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.COMM_NUMERIC_ATTRIBUTE_VALUES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.communication_constraint_rhs)) %then %do;
      %mo_create_tables(lib=mometa,table=communication_constraint_rhs,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.COMMUNICATION_CONSTRAINT_RHS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.customer_attributes)) %then %do;
      %mo_create_tables(lib=mometa,table=customer_attributes,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.CUSTOMER_ATTRIBUTES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.customer_attribute_values)) %then %do;
      %mo_create_tables(lib=mometa,table=customer_attribute_values,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.CUSTOMER_ATTRIBUTE_VALUES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.customer_offer_attributes)) %then %do;
      %mo_create_tables(lib=mometa,table=customer_offer_attributes,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.CUSTOMER_OFFER_ATTRIBUTES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.channels)) %then %do;
      %mo_create_tables(lib=mometa,table=channels,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.CHANNELS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.channel_constraint_rhs)) %then %do;
      %mo_create_tables(lib=mometa,table=channel_constraint_rhs,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.CHANNEL_CONSTRAINT_RHS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.products)) %then %do;
      %mo_create_tables(lib=mometa,table=products,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.PRODUCTS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.time_periods)) %then %do;
      %mo_create_tables(lib=mometa,table=time_periods,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.TIME_PERIODS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.offer_groups)) %then %do;
      %mo_create_tables(lib=mometa,table=offer_groups,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.OFFER_GROUPS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.offer_subgroups)) %then %do;
      %mo_create_tables(lib=mometa,table=offer_subgroups,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.OFFER_SUBGROUPS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.constraints)) %then %do;
      %mo_create_tables(lib=mometa,table=constraints,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.CONSTRAINTS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.seasonality_contact_policies)) %then %do;
      %mo_create_tables(lib=mometa,table=seasonality_contact_policies,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.SEASONALITY_CONTACT_POLICIES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.rolling_contact_policies)) %then %do;
      %mo_create_tables(lib=mometa,table=rolling_contact_policies,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.ROLLING_CONTACT_POLICIES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.offer_group_contact_policies)) %then %do;
      %mo_create_tables(lib=mometa,table=offer_group_contact_policies,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.OFFER_GROUP_CONTACT_POLICIES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.offer_subgroup_contact_policies)) %then %do;
      %mo_create_tables(lib=mometa,table=offer_subgroup_contact_policies,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.OFFER_SUBGROUP_CONTACT_POLICIES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.market_seg_contact_policies)) %then %do;
      %mo_create_tables(lib=mometa,table=market_seg_contact_policies,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.MARKET_SEG_CONTACT_POLICIES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.default_max_offers_cont_policy)) %then %do;
      %mo_create_tables(lib=mometa,table=default_max_offers_cont_policy,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.DEFAULT_MAX_OFFERS_CONT_POLICY metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.suppression_rules)) %then %do;
      %mo_create_tables(lib=mometa,table=suppression_rules,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.SUPPRESSION_RULES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.suppression_customer_filters)) %then %do;
      %mo_create_tables(lib=mometa,table=suppression_customer_filters,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.SUPPRESSION_CUSTOMER_FILTERS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.suppression_communications)) %then %do;
      %mo_create_tables(lib=mometa,table=suppression_communications,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.SUPPRESSION_COMMUNICATIONS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.offer_template_pre_calc_fields)) %then %do;
      %mo_create_tables(lib=mometa,table=offer_template_pre_calc_fields,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.OFFER_TEMPLATE_PRE_CALC_FIELDS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.offer_template_calc_fields)) %then %do;
      %mo_create_tables(lib=mometa,table=offer_template_calc_fields,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.OFFER_TEMPLATE_CALC_FIELDS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.offer_details_calc_fields)) %then %do;
      %mo_create_tables(lib=mometa,table=offer_details_calc_fields,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.OFFER_DETAILS_CALC_FIELDS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.channel_filters)) %then %do;
      %mo_create_tables(lib=mometa,table=channel_filters,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.CHANNEL_FILTERS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.offer_filters)) %then %do;
      %mo_create_tables(lib=mometa,table=offer_filters,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.OFFER_FILTERS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.customer_filters)) %then %do;
      %mo_create_tables(lib=mometa,table=customer_filters,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.CUSTOMER_FILTERS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.reportvars)) %then %do;
      %mo_create_tables(lib=mometa,table=reportvars,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.REPORTVARS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.reports)) %then %do;
      %mo_create_tables(lib=mometa,table=reports,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.REPORTS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.report_filters)) %then %do;
      %mo_create_tables(lib=mometa,table=report_filters,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.REPORT_FILTERS metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.report_filter_values)) %then %do;
      %mo_create_tables(lib=mometa,table=report_filter_values,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.REPORT_FILTER_VALUES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.reports_compare)) %then %do;
      %mo_create_tables(lib=mometa,table=reports_compare,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.REPORTS_COMPARE metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;
   %if not %sysfunc(exist(mometa.properties)) %then %do;
      %mo_create_tables(lib=mometa,table=properties,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 104;
         %put %mo_error Error encountered creating the MOMETA.PROPERTIES metadata table.;
         %goto ERROR_EXIT;
      %end;
   %end;


   %*--------------------------------;                                                                                                                            
   %* Strip username of bad chars    ;
   %*--------------------------------;
   %local authenticated_username;
   %let authenticated_username=&username;
   %let username=%sysfunc(translate(&username,___,/\@));      


   %*--------------------------------;                                                                                                                            
   %* Check for unique username      ;
   %*--------------------------------;      
   %if (%mo_obs_in(mometa.users)) %then %do;  
      %mo_no_notes(off)
      proc sql noprint;
         select username
         into :users separated by ' '
         from mometa.users;
      quit;
      %mo_no_notes(on)
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error occurred checking for unique username (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
      %if %sysfunc(indexw(%str(%upcase(&users)),%bquote(%upcase(&username)))) %then %do;
         %put %mo_error Username = %upcase(&username) already defined in the MOMETA.USERS table.;
         %let &_rc = 110;       
         %goto ERROR_EXIT;
      %end;  
   %end;   
   

   %*--------------------------------;                                                                                                                            
   %* Create user directory          ;
   %*--------------------------------;  
   %let dir = %mo_pathname(&moproj.&separator.&username);
   %if (%sysfunc(fileexist("&dir"))) %then %do;     
      %mo_delete_file_item(item=&dir,_rc=dirrc)
      %if (&dirrc ne 0) %then %do;
         %let &_rc = 115;        
         %put %mo_error I/O error encountered deleting &dir for user = %upcase(&username) (dirrc=&dirrc).;  
         %goto ERROR_EXIT;                                                                                    
      %end;  
   %end;

   %let parent_dir=&moproj;   
   %let new_dir=%mo_pathname(&username);

   %global cdrc;
   %let cdrc=;      
   dm 'af catalog=sashelp.moutil.create_directory.scl';
   %if (%bquote(&cdrc) eq %str()) %then %do;                                                                           
      %let &_rc = 105;                                                                                     
      %put %mo_error I/O error encountered creating &dir for user = %upcase(&username).;  
      %goto ERROR_EXIT;                                                                                    
   %end;                                                                                                   


   %*--------------------------------;                                                                                                                            
   %* Create user metadata           ;
   %*--------------------------------;      
   %mo_create_tables(lib=work,table=users,_rc=cudrc)
   %if (&cudrc ne 0) %then %do;
      %let &_rc = 100; 
      %put %mo_error Error occurred creating WORK.USERS table.;    
      %goto ERROR_EXIT;   
   %end;
   %mo_lock_dataset(data=mometa.uid,_rc=lckrc)
   %if (&lckrc ne 0) %then %do;
      %put %mo_error Error obtaining lock of MOMETA.UID.;
      %let &_rc = 106; 
      %goto ERROR_EXIT;
   %end;
   %mo_gen_uid(r_macro=user_id, data=mometa.users, id_var=user_id, _rc=uidrc)   
   %if (&uidrc ne 0) %then %do;
      %let &_rc = 108;                                                                                     
      %put %mo_error Could not generate a unique user_id.;  
      %goto ERROR_EXIT;  
   %end; 
   %mo_no_notes(off)


   %*--------------------------------;                                                                                                                            
   %* Add user                       ;
   %*--------------------------------;      
   proc sql;
      insert into work.users
      set 
         user_id     =  &user_id,
         authenticated_username    = "&authenticated_username",      
         username    = "&username",      
         password    = "",
         name        = "&name",
         job_title   = "&job_title",
         description = "&description",
         email       = "&email",
         phone       = "&phone",
         manage_project_input  = "%upcase(&manage_project_input)",
         manage_user_profiles  = "%upcase(&manage_user_profiles)",
         manage_projects       = "%upcase(&manage_projects)",
         profile_created_by    = "&profile_created_by"
         ; 
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;     
      %put %mo_error SQL error encountered while updating WORK.USERS table (sqlrc=&sqlrc).;  
      %goto ERROR_EXIT;
   %end;
   %mo_no_notes(on)
   
   %mo_record_delete(metadata_table = mometa.users
                    ,where          = user_id eq &user_id
                    ,_rc            = delrc);
   %if (&delrc ne 0) %then %do;
      %let &_rc = 103;
      %goto ERROR_EXIT;
   %end;

   %mo_record_insert(metadata_table = mometa.users
                    ,update_table   = work.users
                    ,by             = user_id
                    ,_rc            = updrc);
   %if (&updrc ne 0) %then %do;
      %let &_rc = 103;
      %goto ERROR_EXIT;
   %end;


   %*--------------------------------;                                                                                                                            
   %* Init noshare list              ;
   %*--------------------------------;      
   %mo_create_tables(lib=work,table=users_noshare,_rc=cudrc)
   %if (&cudrc ne 0) %then %do;
      %put %mo_error Error occurred creating WORK.USERS_NOSHARE table.;    
      %let &_rc = 200;
   %end;
   %mo_no_notes(off)
   data work.users_noshare (keep=user_id noshare);
      set mometa.users;
      where user_id ne &user_id;
      noshare = &user_id;
      output;
      noshare = user_id;
      user_id = &user_id;
      output;
   run;
   %mo_no_notes(on)
   %if (&syserr gt 4) %then %do;   
      %put %mo_error Error occurred updating WORK.USERS_NOSHARE table (syserr=&syserr).;    
      %let &_rc = 200;
   %end;
   
   %mo_record_delete(metadata_table = mometa.users_noshare
                    ,where          = user_id eq &user_id or noshare eq &user_id
                    ,_rc            = delrc);
   %if (&delrc ne 0) %then %do;
      %let &_rc = 200;
   %end;

   %mo_record_insert_multiple(metadata_table = mometa.users_noshare
                             ,update_table   = work.users_noshare
                             ,by             = user_id noshare
                             ,_rc            = updrc);
   %if (&updrc ne 0) %then %do;
      %let &_rc = 200;
   %end;


   %*--------------------------------;                                                                                                                            
   %* Create mo_opt_config.sas       ;
   %*--------------------------------;   
   %mo_opt_config(path=&moproj, _rc=ocrc)   
   %if (&ocrc ne 0) %then %do;
      %let &_rc = 200;
   %end;


   %*--------------------------------;                                                                                                                            
   %* Exit processing                ;                                                                                                                            
   %*--------------------------------;    
   %if (&&&_rc eq 0) %then                                                                                                                         
      %put %mo_note Successfully created user = %upcase(&username) with user_id = &user_id..; 
   %else
      %put %mo_warning Created user = %upcase(&username) with user_id = &user_id. with abnormal conditions (rc=&&&_rc).; 
   %put;
   %goto EXIT;          
   
   %ERROR_EXIT:
   %put;
   %put %mo_error Error encountered during creation of user = %upcase(&username) (rc=&&&_rc).;    
   %put %mo_error User = %upcase(&username) not created.;   
   %put;                                                                
        
        
   %EXIT:    
   %mo_unlock_dataset(data=mometa.uid)    

    
%mend mo_create_user;
