/************************************************************************/
/* Copyright (c) 2003-2005 by SAS Institute Inc., Cary, NC 27513, USA   */
/*                       All Rights Reserved.                           */   
/*                                                                      */        
/* Name:    mo_edit_user                                                */
/*                                                                      */
/* Purpose: Update a MO user definition                                 */
/*                                                                      */
/* Input:   user_id              - unique userid                        */
/*          username             - MO username              (optional)  */
/*          password             - MO password              (optional)  */
/*          name                 - user's name              (optional)  */
/*          job_title            - user's job title         (optional)  */
/*          description          - description              (optional)  */
/*          email                - user's email address     (optional)  */
/*          phone                - user's phone number      (optional)  */
/*          manage_project_input - Y=yes, N=no              (optional)  */
/*          manage_user_profiles - Y=yes, N=no              (optional)  */
/*          manage_projects      - Y=yes, N=no              (optional)  */
/*          profile_create_by    - user who created profile (optional)  */
/*                                                                      */  
/* Output:  &_rc = return code                                          */
/*                                                                      */  
/* Return Codes:                                                        */  
/*                                                                      */  
/*          100   generic error                                         */
/*          101   invalid metadata detected                             */
/*          102   invalid or missing input parameter value              */
/*          103   I/O error updating the USERS table                    */
/*          106   error obtaining lock                                  */
/*          107   SQL error                                             */
/*          110   username not unique                                   */
/*          112   I/O error renaming user directory                     */
/*                                                                      */
/* HISTORY:                                                             */
/* 30Jan04 S0237007 Creation (dwh)                                      */  
/* 31Aug04 S0266519 Overwrite non-blank descriptions with blank (dwh)   */
/*                                                                      */
/* MO 4.1 Hotfix:                                                       */
/* 17Feb05 S0284290 Use authenticated username (dwh)                    */
/*                                                                      */
/************************************************************************/

%macro mo_edit_user(user_id               = 
                   ,username              =
                   ,password              =
                   ,name                  =
                   ,job_title             =
                   ,description           =
                   ,email                 =
                   ,phone                 =
                   ,manage_project_input  =
                   ,manage_user_profiles  =
                   ,manage_projects       =
                   ,profile_created_by    =
                   ,_rc        =
                   );

   %local macname;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);
	
   %if %bquote(&_rc) eq %str() %then %do;                                                                                                  
      %local rc;                                                                                                                      
      %let _rc = rc;                                                                                                                    
   %end;                                                                                                                                   
   %else %global &_rc;                                                                                                                     
   %let &_rc = 0;       
   
   %if (%bquote(&user_id) eq %str()) %then %do;
      %put %mo_error No user_id was specified.;
      %let &_rc = 102;       
      %goto ERROR_EXIT;
   %end;   


   %*--------------------------------;                                                                                                                            
   %* Obtain lock for metadata update;
   %*--------------------------------;      
   %mo_lock_dataset(data=mometa.uid,_rc=lckrc)
   %if (&lckrc ne 0) %then %do;
      %put %mo_error Error obtaining lock of MOMETA.UID.;
      %let &_rc = 106; 
      %goto ERROR_EXIT;
   %end;


   %*--------------------------------;                                                                                                                            
   %* Retrieve metadata              ;
   %*--------------------------------;  
   %mo_no_notes(off)
   %local old_username;
   data _null_;
      set mometa.users;
      if (user_id eq &user_id) then do; 
         call symput('old_username',strip(username));
      end;
   run;
   %if (%bquote(&old_username) eq %str()) %then %do;
      %let &_rc = 101;
      %put %mo_error Invalid metadata detected.;  
      %put %mo_error Username not identified for user_id = &user_id..;  
      %goto ERROR_EXIT;
   %end;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;    
      %put %mo_error Error occured while reading metadata (syserr=&syserr).;   
      %goto ERROR_EXIT; 
   %end;      
   %mo_no_notes(on)      


   %*--------------------------------;                                                                                                                            
   %* Strip username of bad chars    ;
   %*--------------------------------;
   %local authenticated_username;
   %let authenticated_username=&username;
   %let username=%sysfunc(translate(&username,___,/\@));      


   %*--------------------------------;                                                                                                                            
   %* Check for unique username      ;
   %*--------------------------------;   
   %mo_no_notes(off)
   %if (%bquote(&username) ne %str()) %then %do;
      %local users;
      proc sql noprint;
         select username
         into :users separated by ' '
         from mometa.users
         where (user_id ne &user_id);
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error occurred checking for unique username (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
      %if %sysfunc(indexw(%str(&users),%bquote(&username))) %then %do;
         %put %mo_error Username = %upcase(&username) already defined in the MOMETA.USERS table.;
         %let &_rc = 110;       
         %goto ERROR_EXIT;
      %end;  
   %end; 
   %else %let username = &old_username;
   %mo_no_notes(on)


   %*--------------------------------;                                                                                                                            
   %* Rename users directory         ;
   %*--------------------------------;   
   %if (%bquote(&username) ne %str() and %bquote(&username) ne %str(&old_username)) %then %do;  
      %let old_dir = %mo_pathname(&moproj.&separator.&old_username);  
      %let new_dir = %mo_pathname(&moproj.&separator.&username.);  

      %global rdrc;
      %let rdrc=-1;   
      dm 'af catalog=sashelp.moutil.rename_directory.scl';
      %if (%bquote(&rdrc) ne 0) %then %do;                                                                           
         %let &_rc = 112;                                                                                     
         %put %mo_error I/O error encountered renaming user directory from &old_dir to &new_dir..;  
         %goto ERROR_EXIT;                                                                                    
      %end;                                                                  
   %end;


   %*--------------------------------;                                                                                                                            
   %* Update USERS table             ;
   %*--------------------------------;      
   %mo_create_tables(lib=work, table=users, _rc=cudrc)
   %if (&cudrc ne 0) %then %do;
      %let &_rc = 100; 
      %put %mo_error Error occurred creating WORK.USERS table.;    
      %goto ERROR_EXIT;   
   %end;
   %mo_no_notes(off)
   data work.users;
      set mometa.users;
      where user_id eq &user_id;

      authenticated_username = "&authenticated_username";      
      username="&username"; 
      password="";
      name="&name";
      job_title="&job_title";
      description="&description";
      email="&email";
      phone="&phone";
      manage_project_input="&manage_project_input"; 
      manage_user_profiles="&manage_user_profiles";
      manage_projects="&manage_projects";
      profile_created_by="&profile_created_by"; 
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;     
      %put %mo_error Error encountered while updating WORK.USERS table (syserr=&syserr).;  
      %goto ERROR_EXIT;
   %end;
   %mo_no_notes(on)
   
   %mo_record_update(metadata_table = mometa.users
                    ,update_table   = work.users
                    ,by             = user_id
                    ,updatemode     = nomissingcheck
                    ,_rc            = updrc);
   %if (&updrc ne 0) %then %do;
      %let &_rc = 103;
      %goto ERROR_EXIT;
   %end;


   %*--------------------------------;                                                                                                                            
   %* Exit processing                ;                                                                                                                            
   %*--------------------------------;                                                                   
   %* Successful completion;                                                                                                                            
   %put %mo_note Successfully updated profile for user = %upcase(&username) with user_id = &user_id..; 
   %goto EXIT;          

   
   %ERROR_EXIT:
   %put;
   %put %mo_error Error encountered during update of profile for user_id = &user_id (rc=&&&_rc).;                                                                       
      
      
   %EXIT:   
   %mo_unlock_dataset(data=mometa.uid)    
    
    
%mend mo_edit_user;
